package connection

import (
	"testing"
	"time"

	omd "git.loafle.net/overflow/model/discovery"
	omm "git.loafle.net/overflow/model/meta"
	"git.loafle.net/overflow_scanner/probe/discovery/session"
	"git.loafle.net/overflow_scanner/probe/discovery/types"
)

func TestScan(t *testing.T) {
	s := session.NewMockDiscoverySession(
		"testRequester",
		types.DiscoveryRequestTypeHost,
		&omd.Zone{
			Network:    "192.168.1.0/24",
			Iface:      "enp3s0",
			MetaIPType: omm.ToMetaIPType(omm.MetaIPTypeEnumV4),
			Address:    "192.168.1.101",
			Mac:        "44:8a:5b:f1:f1:f3",
		},
		&omd.DiscoverHost{
			MetaIPType:      omm.ToMetaIPType(omm.MetaIPTypeEnumV4),
			FirstScanRange:  "192.168.1.1",
			LastScanRange:   "192.168.1.254",
			DiscoveryConfig: &omd.DiscoveryConfig{},
			DiscoverPort: &omd.DiscoverPort{
				FirstScanRange: 1,
				LastScanRange:  65535,
				IncludeTCP:     true,
			},
		},
	)

	targetHost := &omd.Host{
		MetaIPType: omm.ToMetaIPType(omm.MetaIPTypeEnumV4),
		Name:       "atGame",
		Address:    "192.168.1.1",
		Mac:        "00:11:32:7f:20:61",
		Zone:       s.Zone(),
	}

	type args struct {
		discoverySession session.DiscoverySession
		targetHost       *omd.Host
	}
	tests := []struct {
		name    string
		args    args
		wantErr bool
	}{
		{
			name: "1",
			args: args{
				discoverySession: s,
				targetHost:       targetHost,
			},
			wantErr: false,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if err := Scan(tt.args.discoverySession, tt.args.targetHost); (err != nil) != tt.wantErr {
				t.Errorf("Scan() error = %v, wantErr %v", err, tt.wantErr)
			}
		})
	}
}

func Test_scanPort(t *testing.T) {
	type args struct {
		discoverySession session.DiscoverySession
		ports            map[int]*omd.Port
		targetHost       *omd.Host
		port             int
		timeout          time.Duration
	}
	tests := []struct {
		name string
		args args
	}{
		// TODO: Add test cases.
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			scanPort(tt.args.discoverySession, tt.args.ports, tt.args.targetHost, tt.args.port, tt.args.timeout)
		})
	}
}

func TestUlimit(t *testing.T) {
	tests := []struct {
		name string
		want int64
	}{
		// TODO: Add test cases.
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := Ulimit(); got != tt.want {
				t.Errorf("Ulimit() = %v, want %v", got, tt.want)
			}
		})
	}
}
